/*
 * JFrameOracle.java
 *
 * Created on 23 de marzo de 2005, 16:20
 */

package learningbom;

import java.awt.Dimension;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowFocusListener;
import java.awt.event.WindowListener;
import javax.swing.JFrame;
import javax.swing.JPanel;


/**
 *
 * @author Alejandro Touza Paredes
 */
public class ConsoleBOM extends javax.swing.JFrame {
    
    private MyCanvasBOM c;
    private MyCanvasOracle co;
    private String pattern="";
    private JFrame jFrameMyCanvasBOM, jFrameMyCanvasOracle;
    private JPanel jPanelMyCanvasBOM, jPanelMyCanvasOracle;
    
    /** Crea una nueva instancia de ConsoleBOM. */
    public ConsoleBOM() {
        
        initConsoleBOM();
        show();
        initMyCanvasBOM();
        initMyCanvasOracle();
        
        jTextFieldText.requestFocus();
    }
    
    /** Crea una nueva instancia de ConsoleBOM que contiene como pattern a buscar el inverso
     *  del pasado. */
    public ConsoleBOM(String pattern) {
        
        initConsoleBOM();
        show();
        initMyCanvasBOM();
        initMyCanvasOracle();
        
        jTextFieldPattern.setText( (new StringBuffer(pattern)).reverse().toString());
        jTextFieldText.requestFocus();
    }
    
    private void initConsoleBOM(){
        
        setIconImage(Resources.createImageIcon("/images/icons/IconoProgramaConsolaBOM.gif").getImage());
        initComponents();
        //Repintamos el Canvas cada vez que se gane el foco
        this.addWindowFocusListener(new WindowFocusListener(){
            public void windowLostFocus(WindowEvent evt){}
            public void windowGainedFocus(WindowEvent evt){windowGainedFocusEvent(evt);}
        });
        this.addWindowListener(new WindowListener(){
            public void windowOpened(WindowEvent evt){}
            public void windowIconified(WindowEvent evt)
            {
                jFrameMyCanvasBOM.hide();
                if(jCheckBoxShowGraph.isSelected())jFrameMyCanvasOracle.hide();
            }
            public void windowDeiconified(WindowEvent evt)
            {
                jFrameMyCanvasBOM.show();
                if(jCheckBoxShowGraph.isSelected())jFrameMyCanvasOracle.show();
                toFront();
            }
            public void windowDeactivated(WindowEvent evt){}
            public void windowClosing(WindowEvent evt){doClose();}
            public void windowClosed(WindowEvent evt){}
            public void windowActivated(WindowEvent evt){}
        });
    }
    
    private void initMyCanvasBOM(){
        
        jFrameMyCanvasBOM = new JFrame();
        jFrameMyCanvasBOM.setIconImage(Resources.createImageIcon("/images/icons/IconoProgramaConsolaBOM32x32.gif").getImage());
        jFrameMyCanvasBOM.setBounds(this.getX() + this.getWidth(), this.getY(),
                                    SystemScreenSize().width - this.getWidth(), 
                                    SystemScreenSize().height - 30 - 250);        
        jFrameMyCanvasBOM.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        jFrameMyCanvasBOM.setVisible(true); //necesario para contener al Canvas
        jFrameMyCanvasBOM.setTitle("Backwards Oracle Matching");
        
        JPanel jPanelMyCanvasBOM = new JPanel();
        jPanelMyCanvasBOM = (JPanel)jFrameMyCanvasBOM.getContentPane();
        jPanelMyCanvasBOM.setPreferredSize(jFrameMyCanvasBOM.getSize());
        jPanelMyCanvasBOM.setLayout(null);
        
        c = new MyCanvasBOM(jPanelMyCanvasBOM); //Ahora el JPanel Contiene el canvas
        jPanelMyCanvasBOM.setBackground(c.getBackground());
        jFrameMyCanvasBOM.setBackground(c.getBackground());
        
        c.setDefaultListeners(jFrameMyCanvasBOM, jPanelMyCanvasBOM);
        c.addKeyListener(new KeyListener(){
            public void keyTyped(KeyEvent evt){} 
            public void keyReleased(KeyEvent evt){}
            public void keyPressed(KeyEvent evt){ MyCanvasBOMKeyPressed(evt);}
        });
        jFrameMyCanvasBOM.addKeyListener(new KeyListener(){
            public void keyTyped(KeyEvent evt){ }
            public void keyReleased(KeyEvent evt){}
            public void keyPressed(KeyEvent evt){ MyCanvasBOMKeyPressed(evt);}
        });
    }
    private void initMyCanvasOracle(){
        
        jFrameMyCanvasOracle = new JFrame();
        jFrameMyCanvasOracle.setIconImage(Resources.createImageIcon("/images/icons/IconoProgramaConsolaBOM32x32.gif").getImage());
        jFrameMyCanvasOracle.setBounds(jFrameMyCanvasBOM.getX(),
                                       jFrameMyCanvasBOM.getY() + jFrameMyCanvasBOM.getHeight(),
                                       jFrameMyCanvasBOM.getWidth(), 
                                       SystemScreenSize().height - jFrameMyCanvasBOM.getHeight() - 30);
                
        jFrameMyCanvasOracle.setDefaultCloseOperation(JFrame.DO_NOTHING_ON_CLOSE);
        
        jFrameMyCanvasOracle.setVisible(true); //necesario para contener al Canvas
        jFrameMyCanvasOracle.setTitle("Factor Oracle");
        
        JPanel jPanelMyCanvasOracle = new JPanel();
        jPanelMyCanvasOracle = (JPanel)jFrameMyCanvasOracle.getContentPane();
        
        jPanelMyCanvasOracle.setPreferredSize(jFrameMyCanvasOracle.getSize());
        jPanelMyCanvasOracle.setLayout(null);
        co = new MyCanvasOracle(jPanelMyCanvasOracle);
        
        co.setSearchMode(true);
        co.setDefaultListeners(jFrameMyCanvasOracle, jPanelMyCanvasOracle);
        co.addKeyListener(new KeyListener(){
               public void keyTyped(KeyEvent evt){ }
               public void keyReleased(KeyEvent evt){}
               public void keyPressed(KeyEvent evt){ MyCanvasOracleKeyPressed(evt);}
        });
        jFrameMyCanvasOracle.addKeyListener(new KeyListener(){
                         public void keyTyped(KeyEvent evt){ }
                         public void keyReleased(KeyEvent evt){}
                         public void keyPressed(KeyEvent evt){ MyCanvasOracleKeyPressed(evt);}
        });
        
    }
    
    // Rutinas que tratan KeyListeners y windowGainedEvent
    private void MyCanvasBOMKeyPressed(KeyEvent evt){
   
        switch(evt.getKeyCode())
        {
            case KeyEvent.VK_SPACE:      if(jButtonStep.isEnabled()) doStep();
                                         break;
            case KeyEvent.VK_BACK_SPACE: if(jButtonStepBack.isEnabled()) doStepBack();
                                         break;
            case KeyEvent.VK_ENTER:      if(jButtonComplete.isEnabled()) doComplete();
                                         break;
            case KeyEvent.VK_R:          if(jButtonReset.isEnabled()) doReset();
                                         break;
            case KeyEvent.VK_S:          doStart();
                                         break;
            case KeyEvent.VK_PLUS:      
            case KeyEvent.VK_ADD:        doBigger();
                                         break;
            case KeyEvent.VK_MINUS:      
            case KeyEvent.VK_SUBTRACT:   doSmaller();
                                         break;
            case KeyEvent.VK_F:          jCheckBoxFitToScreen.doClick();
                                         break;
            case KeyEvent.VK_C:          if(jButtonMoveToStart.isEnabled()) doMoveToStart();
                                         break;
            case KeyEvent.VK_H:                             
            case KeyEvent.VK_G:          jCheckBoxShowGraph.doClick();
                                         //c.requestFocus();
                                         break;
            case KeyEvent.VK_LEFT:       doMove(150, 0);
                                         break;
            case KeyEvent.VK_RIGHT:      doMove(-150, 0);
                                         break;
            case KeyEvent.VK_UP:         doMove(0, 150);
                                         break;
            case KeyEvent.VK_DOWN:       doMove(0, -150);
                                         break;                               
        }
   }
    private void MyCanvasOracleKeyPressed(KeyEvent evt){
   
     if(jCheckBoxShowGraph.isSelected() && c.isStarted())
     {
        switch(evt.getKeyCode())
        {
            case KeyEvent.VK_SPACE:      if(jButtonStep.isEnabled()) doStep();
                                         break;
            case KeyEvent.VK_BACK_SPACE: if(jButtonStepBack.isEnabled()) doStepBack();
                                         break;
            case KeyEvent.VK_ENTER:      if(jButtonComplete.isEnabled()) doComplete();
                                         break;
            case KeyEvent.VK_R:          if(jButtonReset.isEnabled()) doReset();
                                         break;
            case KeyEvent.VK_S:          doStart();
                                         break;
            case KeyEvent.VK_H:                             
            case KeyEvent.VK_G:          jCheckBoxShowGraph.doClick();
                                         if(!jCheckBoxShowGraph.isSelected()) c.requestFocus();
                                         break;
            case KeyEvent.VK_PLUS: 
            case KeyEvent.VK_ADD:        co.makeItBigger();
                                         co.repaint();
                                         break;
            case KeyEvent.VK_MINUS:      
            case KeyEvent.VK_SUBTRACT:   co.makeItSmaller();
                                         co.repaint();
                                         break;
            case KeyEvent.VK_F:          co.fitToScreen();
                                         co.repaint();
                                         break;
            case KeyEvent.VK_C:          co.moveToDefaultStartPoint();
                                         co.repaint();
                                         break;
            case KeyEvent.VK_LEFT:       co.moveFromStartPoint(100,  0);
                                         co.repaint();
                                         break;
            case KeyEvent.VK_RIGHT:      co.moveFromStartPoint(-100 ,  0);
                                         co.repaint();
                                         break;
            case KeyEvent.VK_UP:         co.moveFromStartPoint(0  ,100);
                                         co.repaint();
                                         break;
            case KeyEvent.VK_DOWN:       co.moveFromStartPoint(0  , -100);
                                         co.repaint();
                                         break;                                 
        }
        
     }
   }
    private void windowGainedFocusEvent(WindowEvent evt){
       
       if(c.isStarted())
       {
           c.repaint();
           if(jCheckBoxShowGraph.isSelected()) co.repaint();
       }
   }
   
    private Dimension SystemScreenSize() {   
        
        return Toolkit.getDefaultToolkit().getScreenSize();
   }  
    /** This method is called from within the constructor to
     * initialize the form.
     * WARNING: Do NOT modify this code. The content of this method is
     * always regenerated by the Form Editor.
     */
    private void initComponents() {//GEN-BEGIN:initComponents
        java.awt.GridBagConstraints gridBagConstraints;

        jPanelConsole = new javax.swing.JPanel();
        jPanelBOM = new javax.swing.JPanel();
        jTextFieldText = new javax.swing.JTextField();
        jTextFieldPattern = new javax.swing.JTextField();
        jButtonStart = new javax.swing.JButton();
        jButtonReset = new javax.swing.JButton();
        jPanelSteps = new javax.swing.JPanel();
        jButtonStepBack = new javax.swing.JButton();
        jButtonStep = new javax.swing.JButton();
        jButtonComplete = new javax.swing.JButton();
        jPanelRedraw = new javax.swing.JPanel();
        jTextFieldSize = new javax.swing.JTextField();
        jButtonResizeOK = new javax.swing.JButton();
        jButtonResizeBigger = new javax.swing.JButton();
        jButtonResizeSmaller = new javax.swing.JButton();
        jCheckBoxFitToScreen = new javax.swing.JCheckBox();
        jButtonMoveToStart = new javax.swing.JButton();
        jCheckBoxShowGraph = new javax.swing.JCheckBox();
        jPanelHelp = new javax.swing.JPanel();
        jButtonHelp = new javax.swing.JButton();

        getContentPane().setLayout(new java.awt.GridLayout(1, 0));

        setDefaultCloseOperation(javax.swing.WindowConstants.DO_NOTHING_ON_CLOSE);
        setTitle("Consola BOM");
        setCursor(new java.awt.Cursor(java.awt.Cursor.DEFAULT_CURSOR));
        setName("JFrameConsole");
        getAccessibleContext().setAccessibleDescription("Consola para Grafos Oracle");
        jPanelConsole.setBorder(new javax.swing.border.TitledBorder("Consola"));
        jPanelConsole.setMinimumSize(new java.awt.Dimension(40, 0));
        jPanelConsole.setPreferredSize(new java.awt.Dimension(130, 350));
        jPanelBOM.setLayout(new java.awt.GridLayout(6, 1));

        jPanelBOM.setBorder(new javax.swing.border.TitledBorder("BOM"));
        jTextFieldText.setText("GTACTAGAATGTGTAGACATGTATGGTG");
        jTextFieldText.setMargin(new java.awt.Insets(1, 1, 2, 4));
        jTextFieldText.setMaximumSize(new java.awt.Dimension(50, 20));
        jTextFieldText.setPreferredSize(new java.awt.Dimension(100, 20));
        jTextFieldText.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jTextFieldTextActionPerformed(evt);
            }
        });

        jPanelBOM.add(jTextFieldText);

        jTextFieldPattern.setText("ATGTATG");
        jTextFieldPattern.setMargin(new java.awt.Insets(1, 1, 2, 4));
        jTextFieldPattern.setMaximumSize(new java.awt.Dimension(50, 20));
        jTextFieldPattern.setPreferredSize(new java.awt.Dimension(100, 20));
        jPanelBOM.add(jTextFieldPattern);

        jButtonStart.setText("Empezar");
        jButtonStart.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonStart.setMaximumSize(getPreferredSize());
        jButtonStart.setMinimumSize(getPreferredSize());
        jButtonStart.setPreferredSize(new java.awt.Dimension(100, 23));
        jButtonStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStartActionPerformed(evt);
            }
        });

        jPanelBOM.add(jButtonStart);

        jButtonReset.setText("Resetear");
        jButtonReset.setEnabled(false);
        jButtonReset.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonReset.setMaximumSize(getPreferredSize());
        jButtonReset.setMinimumSize(getPreferredSize());
        jButtonReset.setPreferredSize(new java.awt.Dimension(100, 23));
        jButtonReset.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResetActionPerformed(evt);
            }
        });

        jPanelBOM.add(jButtonReset);

        jPanelSteps.setLayout(new java.awt.GridLayout(1, 2));

        jButtonStepBack.setText("<<");
        jButtonStepBack.setEnabled(false);
        jButtonStepBack.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonStepBack.setMaximumSize(getPreferredSize());
        jButtonStepBack.setMinimumSize(getPreferredSize());
        jButtonStepBack.setPreferredSize(new java.awt.Dimension(50, 23));
        jButtonStepBack.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStepBackActionPerformed(evt);
            }
        });

        jPanelSteps.add(jButtonStepBack);

        jButtonStep.setText(">>");
        jButtonStep.setEnabled(false);
        jButtonStep.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonStep.setMaximumSize(getPreferredSize());
        jButtonStep.setMinimumSize(getPreferredSize());
        jButtonStep.setPreferredSize(new java.awt.Dimension(50, 23));
        jButtonStep.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonStepActionPerformed(evt);
            }
        });

        jPanelSteps.add(jButtonStep);

        jPanelBOM.add(jPanelSteps);

        jButtonComplete.setText("Completar");
        jButtonComplete.setEnabled(false);
        jButtonComplete.setMargin(new java.awt.Insets(0, 0, 0, 0));
        jButtonComplete.setMaximumSize(getPreferredSize());
        jButtonComplete.setMinimumSize(getPreferredSize());
        jButtonComplete.setPreferredSize(new java.awt.Dimension(100, 23));
        jButtonComplete.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonCompleteActionPerformed(evt);
            }
        });

        jPanelBOM.add(jButtonComplete);

        jPanelConsole.add(jPanelBOM);

        jPanelRedraw.setLayout(new java.awt.GridBagLayout());

        jPanelRedraw.setBorder(new javax.swing.border.TitledBorder("Opciones"));
        jPanelRedraw.setMaximumSize(getPreferredSize());
        jTextFieldSize.setHorizontalAlignment(javax.swing.JTextField.CENTER);
        jTextFieldSize.setText("30");
        jTextFieldSize.setEnabled(false);
        jTextFieldSize.setMaximumSize(getPreferredSize());
        jTextFieldSize.setMinimumSize(getPreferredSize());
        jTextFieldSize.setPreferredSize(new java.awt.Dimension(60, 20));
        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 1;
        jPanelRedraw.add(jTextFieldSize, gridBagConstraints);

        jButtonResizeOK.setText("OK");
        jButtonResizeOK.setEnabled(false);
        jButtonResizeOK.setMargin(new java.awt.Insets(2, 0, 2, 0));
        jButtonResizeOK.setMaximumSize(getPreferredSize());
        jButtonResizeOK.setMinimumSize(getPreferredSize());
        jButtonResizeOK.setPreferredSize(new java.awt.Dimension(60, 23));
        jButtonResizeOK.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResizeOKActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 2;
        jPanelRedraw.add(jButtonResizeOK, gridBagConstraints);

        jButtonResizeBigger.setText("+");
        jButtonResizeBigger.setEnabled(false);
        jButtonResizeBigger.setMargin(new java.awt.Insets(2, 0, 2, 0));
        jButtonResizeBigger.setMaximumSize(getPreferredSize());
        jButtonResizeBigger.setMinimumSize(getPreferredSize());
        jButtonResizeBigger.setPreferredSize(new java.awt.Dimension(40, 23));
        jButtonResizeBigger.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResizeBiggerActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 1;
        jPanelRedraw.add(jButtonResizeBigger, gridBagConstraints);

        jButtonResizeSmaller.setText("-");
        jButtonResizeSmaller.setEnabled(false);
        jButtonResizeSmaller.setMargin(new java.awt.Insets(2, 0, 2, 0));
        jButtonResizeSmaller.setMaximumSize(getPreferredSize());
        jButtonResizeSmaller.setMinimumSize(getPreferredSize());
        jButtonResizeSmaller.setPreferredSize(new java.awt.Dimension(40, 23));
        jButtonResizeSmaller.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonResizeSmallerActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 1;
        gridBagConstraints.gridy = 2;
        jPanelRedraw.add(jButtonResizeSmaller, gridBagConstraints);

        jCheckBoxFitToScreen.setSelected(true);
        jCheckBoxFitToScreen.setText("fit to screen");
        jCheckBoxFitToScreen.setMinimumSize(getPreferredSize());
        jCheckBoxFitToScreen.setPreferredSize(new java.awt.Dimension(100, 23));
        jCheckBoxFitToScreen.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxFitToScreenActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 3;
        gridBagConstraints.gridwidth = java.awt.GridBagConstraints.REMAINDER;
        gridBagConstraints.insets = new java.awt.Insets(5, 0, 0, 0);
        jPanelRedraw.add(jCheckBoxFitToScreen, gridBagConstraints);

        jButtonMoveToStart.setText("Centrar");
        jButtonMoveToStart.setEnabled(false);
        jButtonMoveToStart.setMargin(new java.awt.Insets(2, 0, 2, 0));
        jButtonMoveToStart.setMaximumSize(getPreferredSize());
        jButtonMoveToStart.setMinimumSize(getPreferredSize());
        jButtonMoveToStart.setPreferredSize(new java.awt.Dimension(100, 23));
        jButtonMoveToStart.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jButtonMoveToStartActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridx = 0;
        gridBagConstraints.gridy = 5;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelRedraw.add(jButtonMoveToStart, gridBagConstraints);

        jCheckBoxShowGraph.setSelected(true);
        jCheckBoxShowGraph.setText("show graph");
        jCheckBoxShowGraph.setMinimumSize(getPreferredSize());
        jCheckBoxShowGraph.setPreferredSize(new java.awt.Dimension(100, 23));
        jCheckBoxShowGraph.addActionListener(new java.awt.event.ActionListener() {
            public void actionPerformed(java.awt.event.ActionEvent evt) {
                jCheckBoxShowGraphActionPerformed(evt);
            }
        });

        gridBagConstraints = new java.awt.GridBagConstraints();
        gridBagConstraints.gridy = 4;
        gridBagConstraints.gridwidth = 2;
        gridBagConstraints.fill = java.awt.GridBagConstraints.HORIZONTAL;
        jPanelRedraw.add(jCheckBoxShowGraph, gridBagConstraints);

        jPanelConsole.add(jPanelRedraw);

        jPanelHelp.setLayout(new java.awt.GridLayout(1, 0));

        jPanelHelp.setMaximumSize(getPreferredSize());
        jButtonHelp.setText("Ayuda");
        jButtonHelp.setMargin(new java.awt.Insets(2, 0, 2, 0));
        jButtonHelp.setMaximumSize(new java.awt.Dimension(100, 23));
        jButtonHelp.setMinimumSize(new java.awt.Dimension(100, 23));
        jButtonHelp.setPreferredSize(new java.awt.Dimension(100, 23));
        jPanelHelp.add(jButtonHelp);

        jPanelConsole.add(jPanelHelp);

        getContentPane().add(jPanelConsole);

        setSize(new java.awt.Dimension(138, 442));
    }//GEN-END:initComponents

    //Tratamiento de ActionEvents
    private void jCheckBoxShowGraphActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxShowGraphActionPerformed
        doShowGraph();
    }//GEN-LAST:event_jCheckBoxShowGraphActionPerformed
    private void jButtonStepBackActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStepBackActionPerformed
        doStepBack();
    }//GEN-LAST:event_jButtonStepBackActionPerformed
    private void jTextFieldTextActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jTextFieldTextActionPerformed
        // TODO add your handling code here:
    }//GEN-LAST:event_jTextFieldTextActionPerformed
    private void jButtonResizeBiggerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResizeBiggerActionPerformed
        doBigger();
    }//GEN-LAST:event_jButtonResizeBiggerActionPerformed
    private void jButtonResizeOKActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResizeOKActionPerformed
        doResize();
    }//GEN-LAST:event_jButtonResizeOKActionPerformed
    private void jButtonResizeSmallerActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResizeSmallerActionPerformed
        doSmaller();
    }//GEN-LAST:event_jButtonResizeSmallerActionPerformed
    private void jCheckBoxFitToScreenActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jCheckBoxFitToScreenActionPerformed
        doFitToScreen();
    }//GEN-LAST:event_jCheckBoxFitToScreenActionPerformed
    private void jButtonMoveToStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonMoveToStartActionPerformed
        doMoveToStart();
    }//GEN-LAST:event_jButtonMoveToStartActionPerformed
    private void jButtonStartActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStartActionPerformed
        doStart();
    }//GEN-LAST:event_jButtonStartActionPerformed
    private void jButtonCompleteActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonCompleteActionPerformed
        doComplete();
    }//GEN-LAST:event_jButtonCompleteActionPerformed
    private void jButtonResetActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonResetActionPerformed
        doReset();
    }//GEN-LAST:event_jButtonResetActionPerformed
    private void jButtonStepActionPerformed(java.awt.event.ActionEvent evt) {//GEN-FIRST:event_jButtonStepActionPerformed
        doStep();
    }                                               //Rutinas privadas que atienden los ActionListeners//GEN-LAST:event_jButtonStepActionPerformed
    private void doStart(){
       
        if(jTextFieldText.getText().length()>0 && jTextFieldPattern.getText().length()>0 &&
           jTextFieldText.getText().length()   >= jTextFieldPattern.getText().length())
        {    
            String text = jTextFieldText.getText().toUpperCase();
            pattern = jTextFieldPattern.getText().toUpperCase();
            jTextFieldText.setText(text);
            jTextFieldPattern.setText(pattern);
            
            /// Inicializamos MyCanvasOracle
            String title; 
            String reversedPattern = (new StringBuffer(pattern)).reverse().toString();
            
            if(pattern.length()>30)title = reversedPattern.substring(0, 30) + "..."; 
            else title = reversedPattern.substring(0, pattern.length());
            title+=" (" + pattern.length() + " caracteres)";
            
            jFrameMyCanvasOracle.setTitle("Factor Oracle - " + title);
            co.start(reversedPattern);
            co.fitToScreen();
            co.complete();
            //////////////////////////////////////////////////
            /// Inicializamos MyCanvasBOM
            if(text.length()>25)title = text.substring(0, 25) + "..."; 
            else title = text.substring(0, text.length());
            title+=" (" + text.length() + " caracteres) ";
            if(pattern.length()>8)title += pattern.substring(0, 8) + "..."; 
            else title += pattern.substring(0, pattern.length());
            title+=" (" + pattern.length() + " caracteres)";
            
            jFrameMyCanvasBOM.setTitle("BOM - " + title);
            c.start(text, pattern);
            doFitToScreen();
            ///////////////////////////////////////////////////
            
            jButtonComplete.setEnabled(true);
            jButtonReset.setEnabled(true);
            jButtonStep.setEnabled(true);
            jButtonStepBack.setEnabled(false);
            jButtonResizeBigger.setEnabled(true);
            jButtonResizeSmaller.setEnabled(true);
            jButtonResizeOK.setEnabled(true);
            jTextFieldSize.setEnabled(true);
            jButtonMoveToStart.setEnabled(true);
            
            c.repaint();
            if(jCheckBoxShowGraph.isSelected()) co.repaint();
        }
    }
    private void doComplete(){
        
        c.complete();
      
        jButtonComplete.setEnabled(false);
        jButtonStep.setEnabled(false);
        jButtonStepBack.setEnabled(true);
        
        c.repaint();
        if(jCheckBoxShowGraph.isSelected())
        {
            co.flushTransitionsList();
            co.repaint();
        }
    }
    private void doReset(){
      
        //c.start(c.getText(), pattern);
        
        c.reset();
        
        jButtonComplete.setEnabled(true);
        jButtonStep.setEnabled(true);
        jButtonStepBack.setEnabled(false);
        jButtonResizeBigger.setEnabled(true);
        jButtonResizeSmaller.setEnabled(true);
        jButtonResizeOK.setEnabled(true);
        jTextFieldSize.setEnabled(true);
        jButtonMoveToStart.setEnabled(true);
        
        c.repaint();
        if(jCheckBoxShowGraph.isSelected())
        {
            co.flushTransitionsList();
            co.repaint();
        }
    }
    private void doStep(){
        
        if (!c.step())
        {
            jButtonStep.setEnabled(false);
            jButtonComplete.setEnabled(false);
        }    
        jButtonStepBack.setEnabled(true);
        c.repaint();
        if(jCheckBoxShowGraph.isSelected())
        {    
            co.flushTransitionsList();
            co.addTransitionsFoundToList(c.getCurrentString());
            co.repaint();
        }
    }
    private void doStepBack(){
    
        if (!c.stepBack())
        {
            jButtonStepBack.setEnabled(false);
        }    
        jButtonStep.setEnabled(true);
        jButtonComplete.setEnabled(true);
        c.repaint();
        if(jCheckBoxShowGraph.isSelected())
        {    
            co.flushTransitionsList();
            co.addTransitionsFoundToList(c.getCurrentString());
            co.repaint();
        }
    }
    private void doBigger(){
        
        jTextFieldSize.setText(Integer.toString(c.makeItBigger()));
        if(jCheckBoxFitToScreen.isSelected()) jCheckBoxFitToScreen.doClick();
        c.setFitToScreen(false);
        c.repaint();
    }
    private void doSmaller(){
        
        int fontSize = c.makeItSmaller();
        if (fontSize!=-1)
        {
            c.setFitToScreen(false);
            jTextFieldSize.setText(Integer.toString(fontSize));
            if(jCheckBoxFitToScreen.isSelected()) jCheckBoxFitToScreen.doClick();
            c.repaint();
        }
    }
    private void doResize(){
  
        int size=-1;
        try{ //nos aseguramos que lo que haya en el jTextFieldSize sea un entero
            size = Integer.parseInt(jTextFieldSize.getText());
        }catch(NumberFormatException e){}
        if(size>0 && c.isStarted()){
            c.resizeBOM(size);
            if(jCheckBoxFitToScreen.isSelected()) jCheckBoxFitToScreen.doClick();
            c.setFitToScreen(false);
            c.repaint();
        }
    }
    private void doFitToScreen(){
        
        //Si no hemos hecho un "Empezar" no hacemos la llamada a fitToScreen()
        if(jCheckBoxFitToScreen.isSelected() && c.isStarted()){
            jTextFieldSize.setText(Integer.toString(c.fitToScreen()));
            c.setFitToScreen(true);
            c.repaint();
        }
        else c.setFitToScreen(false);
    }
    private void doShowGraph(){
        
        if(jCheckBoxShowGraph.isSelected())
        {
            if(c.isStarted()) //Comprobamos si hemos empezado (no puede haber solo uno nulo)
            {
                co.flushTransitionsList();
                co.addTransitionsFoundToList(c.getCurrentString());
                co.repaint();
            }
            jFrameMyCanvasOracle.show();
        }
        else jFrameMyCanvasOracle.hide();
    }
    private void doMoveToStart(){
   
        c.moveToDefaultStartPoint();
        c.repaint();
    }
    private void doMove(int x, int y){
        
        if(c.isStarted())
        {
            c.moveFromTextStartPoint(x, y);
            c.repaint();
        }
    }
    private void doClose(){
            
        jFrameMyCanvasBOM.dispose();
        jFrameMyCanvasOracle.dispose();
        dispose();
    }
    ////////////////////////////////////////////
    /**
     * @param args the command line arguments
     */
    public static void main(String args[]) {
        java.awt.EventQueue.invokeLater(new Runnable() {
            public void run() {
                new ConsoleBOM().setVisible(true);
            }
        });
    }
    
    // Variables declaration - do not modify//GEN-BEGIN:variables
    private javax.swing.JButton jButtonComplete;
    private javax.swing.JButton jButtonHelp;
    private javax.swing.JButton jButtonMoveToStart;
    private javax.swing.JButton jButtonReset;
    private javax.swing.JButton jButtonResizeBigger;
    private javax.swing.JButton jButtonResizeOK;
    private javax.swing.JButton jButtonResizeSmaller;
    private javax.swing.JButton jButtonStart;
    private javax.swing.JButton jButtonStep;
    private javax.swing.JButton jButtonStepBack;
    private javax.swing.JCheckBox jCheckBoxFitToScreen;
    private javax.swing.JCheckBox jCheckBoxShowGraph;
    private javax.swing.JPanel jPanelBOM;
    private javax.swing.JPanel jPanelConsole;
    private javax.swing.JPanel jPanelHelp;
    private javax.swing.JPanel jPanelRedraw;
    private javax.swing.JPanel jPanelSteps;
    private javax.swing.JTextField jTextFieldPattern;
    private javax.swing.JTextField jTextFieldSize;
    private javax.swing.JTextField jTextFieldText;
    // End of variables declaration//GEN-END:variables
    
}
